# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.command import Command


class ArchiveCpio(object):
    """
    **Extraction/Creation of cpio archives**

    :param string filename: filename to use for archive extraction or creation
    """
    def __init__(self, filename):
        self.filename = filename

    def create(self, source_dir, exclude=None):
        """
        Create cpio archive

        :param string source_dir: data source directory
        :param list exclude: list of excluded items
        """
        find_excludes = []
        find_command = ['cd', source_dir, '&&', 'find', '.']
        cpio_command = [
            'cpio', '--quiet', '-o', '-H', 'newc', '>', self.filename
        ]
        if exclude:
            for path in exclude:
                if find_excludes:
                    find_excludes.append('-or')
                find_excludes.append('-path')
                find_excludes.append('.' + path)
                find_excludes.append('-prune')
            find_excludes.append('-o')
            find_excludes.append('-print')
            find_command += find_excludes

        bash_command = find_command + ['|'] + cpio_command
        Command.run(
            ['bash', '-c', ' '.join(bash_command)]
        )

    def extract(self, dest_dir):
        """
        Extract cpio archive contents

        :param string dest_dir: target data directory
        """
        bash_command = [
            'cd', dest_dir, '&&',
            'cat', self.filename, '|',
            'cpio', '-i', '--make-directories'
        ]
        Command.run(
            ['bash', '-c', ' '.join(bash_command)]
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import json
import os
from tempfile import NamedTemporaryFile

# project
from kiwi.path import Path
from kiwi.logger import log


class DataOutput(object):
    """
    **Converts dict or list variables to a print friendly json format**

    Output controller for data dictionary. The class expects a
    python dict or list to hold the data to become displayed. So
    far the output format is set to json

    :param data: python dict or list which holds the data
    :type data: dict, list
    :param str style: output style could be either standard or color
    """
    def __init__(self, data, style='standard'):
        self.data = data
        self.style = style
        self.color_json = Path.which('pjson')

    def display(self):
        """
        Show data in json output format and selected style
        """
        self._json()

    def _json(self):
        """
        Show data in json output format and selected style
        """
        if self.style == 'color':
            if self.color_json:
                self._color_json()
            else:
                log.warning('pjson for color output not installed')
                log.warning('run: pip install pjson')
                self._standard_json()
        else:
            self._standard_json()

    def _standard_json(self):
        """
        Show data in plain json output format without any highlighting
        """
        print(
            json.dumps(
                self.data, sort_keys=True, indent=4, separators=(',', ': ')
            )
        )

    def _color_json(self):
        """
        Show data in json output format with nice color highlighting
        """
        out_file = NamedTemporaryFile()
        out_file.write(json.dumps(self.data, sort_keys=True))
        out_file.flush()
        pjson_cmd = ''.join(
            ['cat ', out_file.name, ' | pjson']
        )
        os.system(pjson_cmd)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import importlib
from collections import namedtuple
from xml.etree import ElementTree
from xml.dom import minidom

# project
from kiwi.logger import log

from kiwi.exceptions import (
    KiwiSatSolverPluginError,
    KiwiSatSolverJobError,
    KiwiSatSolverJobProblems
)


class Sat(object):
    """
    **Sat Solver class to run package solver operations**

    The class uses SUSE's libsolv sat plugin
    """
    def __init__(self):
        """
        An instance of Sat auto loads the python solv plugin which is a
        python binding to the libsolv C library. An exception is raised
        if the module failed to load. On success a new solver pool is
        initialized for this instance

        :raises KiwiSatSolverPluginError: if libsolv module can't be loaded
        """
        try:
            self.solv = importlib.import_module('solv')
        except Exception as e:
            raise KiwiSatSolverPluginError(
                '{0}: {1}'.format(type(e).__name__, format(e))
            )

        self.pool = self.solv.Pool()
        self.pool.setarch()

    def add_repository(self, solver_repository):
        """
        Add a repository solvable to the pool. This basically add the
        required repository metadata which is needed to run a solver
        operation later.

        :param object solver_repository: Instance of :class:`SolverRepository`
        """
        solvable = solver_repository.create_repository_solvable()
        pool_repository = self.pool.add_repo(solver_repository.uri.uri)
        pool_repository.add_solv(solvable)
        self.pool.addfileprovides()
        self.pool.createwhatprovides()

    def solve(self, job_names, skip_missing=False, ignore_recommended=True):
        """
        Solve dependencies for the given job list. The list is allowed
        to contain element names of the following format:

        * name
          describes a package name

        * pattern:name
          describes a package collection name whose metadata type
          is called 'pattern' and stored as such in the repository
          metadata. Usually SUSE repos uses that

        * group:name
          describes a package collection name whose metadata type
          is called 'group' and stored as such in the repository
          metadata. Usually RHEL/CentOS/Fedora repos uses that

        :param list job_names: list of strings
        :param bool skip_missing: skip job if not found
        :param bool ignore_recommended: do not include recommended packages

        :raises KiwiSatSolverJobProblems: if solver reports solving problems
        :return: Transaction result information

        :rtype: dict
        """
        solver = self.pool.Solver()
        if ignore_recommended:
            solver.set_flag(self.solv.Solver.SOLVER_FLAG_IGNORE_RECOMMENDED, 1)

        solver_problems = solver.solve(
            self._setup_jobs(job_names, skip_missing)
        )
        solver_problem_info = self._evaluate_solver_problems(
            solver_problems
        )
        if solver_problem_info:
            raise KiwiSatSolverJobProblems(solver_problem_info)

        solver_transaction = solver.transaction()
        return self._evaluate_solver_result(
            solver_transaction
        )

    def _evaluate_solver_problems(self, solver_problems):
        """
        Iterate over solver problems and their solutions

        The method creates a pretty print XML information
        and returns that as a UTF-8 encoded string

        :param object solver_problems: result of :class:`Pool::Solver::solve()`

        :return: solver problem info and solutions

        :rtype: str
        """
        if solver_problems:
            problems = ElementTree.Element('problems')
            for problem in solver_problems:
                problem_detail = ElementTree.SubElement(
                    problems, 'problem', id=format(problem.id),
                    message=problem.findproblemrule().info().problemstr()
                )
                for solution in problem.solutions():
                    solution_detail = ElementTree.SubElement(
                        problem_detail, 'solution', id=format(solution.id)
                    )
                    for option in solution.elements(1):
                        solution_option = ElementTree.SubElement(
                            solution_detail, 'option'
                        )
                        solution_option.text = option.str()

            xml_data_unformatted = ElementTree.tostring(
                problems, 'utf-8'
            )
            xml_data_domtree = minidom.parseString(xml_data_unformatted)
            return xml_data_domtree.toprettyxml(indent="    ")

    def _evaluate_solver_result(self, solver_transaction):
        """
        Iterate over solver result and return a data dictionary

        :param object solver_transaction: result of :class:`Pool::Solver::transaction()`

        :return: dict of packages and their details

        :rtype: dict
        """
        result_type = namedtuple(
            'result_type', [
                'uri', 'installsize_bytes', 'arch', 'version', 'checksum'
            ]
        )
        result = {}
        for solvable in solver_transaction.newpackages():
            name = solvable.lookup_str(self.solv.SOLVABLE_NAME)
            result[name] = result_type(
                uri=solvable.repo.name,
                installsize_bytes=solvable.lookup_num(
                    self.solv.SOLVABLE_INSTALLSIZE
                ),
                arch=solvable.lookup_str(
                    self.solv.SOLVABLE_ARCH
                ),
                version=solvable.lookup_str(
                    self.solv.SOLVABLE_EVR
                ),
                checksum=solvable.lookup_checksum(
                    self.solv.SOLVABLE_CHECKSUM
                )
            )
        return result

    def _setup_jobs(self, job_names, skip_missing):
        """
        Create a solver job list from given list of job names

        :param list job_names: list of package,pattern,group names
        :param bool skip_missing: continue or raise if job selection failed

        :return: list of :class:`Pool.selection()` objects

        :rtype: list
        """
        jobs = []
        for job_name in job_names:
            selection_name = self.solv.Selection.SELECTION_NAME
            selection_provides = self.solv.Selection.SELECTION_PROVIDES
            selection = self.pool.select(
                job_name, selection_name | selection_provides
            )
            if selection.flags() & self.solv.Selection.SELECTION_PROVIDES:
                log.info('--> Using capability match for {0}'.format(job_name))
            if selection.isempty():
                if skip_missing:
                    log.info(
                        '--> Package {0} not found: skipped'.format(job_name)
                    )
                else:
                    raise KiwiSatSolverJobError(
                        'Package {0} not found'.format(job_name)
                    )
            else:
                jobs += selection.jobs(self.solv.Job.SOLVER_INSTALL)

        return jobs



import mock
from mock import patch

from .test_helper import raises

from kiwi.package_manager.base import PackageManagerBase


class TestPackageManagerBase(object):
    def setup(self):
        repository = mock.Mock()
        repository.root_dir = 'root-dir'
        self.manager = PackageManagerBase(repository)

    @raises(NotImplementedError)
    def test_request_package(self):
        self.manager.request_package('name')

    @raises(NotImplementedError)
    def test_request_collection(self):
        self.manager.request_collection('name')

    @raises(NotImplementedError)
    def test_request_product(self):
        self.manager.request_product('name')

    @patch.object(PackageManagerBase, 'request_package_exclusion')
    def test_request_package_lock(self, mock_exclude):
        self.manager.request_package_lock('name')
        mock_exclude.assert_called_once_with('name')

    @raises(NotImplementedError)
    def test_request_package_exclusion(self):
        self.manager.request_package_exclusion('name')

    @raises(NotImplementedError)
    def test_process_install_requests_bootstrap(self):
        self.manager.process_install_requests_bootstrap()

    def test_post_process_install_requests_bootstrap(self):
        self.manager.post_process_install_requests_bootstrap()

    @raises(NotImplementedError)
    def test_process_install_requests(self):
        self.manager.process_install_requests()

    @raises(NotImplementedError)
    def test_process_delete_requests(self):
        self.manager.process_delete_requests()

    @raises(NotImplementedError)
    def test_update(self):
        self.manager.update()

    @raises(NotImplementedError)
    def test_process_only_required(self):
        self.manager.process_only_required()

    @raises(NotImplementedError)
    def test_process_plus_recommended(self):
        self.manager.process_plus_recommended()

    @raises(NotImplementedError)
    def test_match_package_installed(self):
        self.manager.match_package_installed('package_name', 'log')

    @raises(NotImplementedError)
    def test_match_package_deleted(self):
        self.manager.match_package_deleted('package_name', 'log')

    def test_database_consistent(self):
        self.manager.database_consistent()

    def test_dump_reload_package_database(self):
        self.manager.dump_reload_package_database()

    def test_has_failed(self):
        assert self.manager.has_failed(0) is False
        assert self.manager.has_failed(1) is True

    def test_cleanup_requests(self):
        self.manager.cleanup_requests()
        assert self.manager.package_requests == []
        assert self.manager.product_requests == []
        assert self.manager.collection_requests == []
        assert self.manager.exclude_requests == []



from kiwi.bootloader.template.zipl import BootLoaderTemplateZipl


class TestBootLoaderTemplateZipl(object):
    def setup(self):
        self.zipl = BootLoaderTemplateZipl()

    def test_get_template(self):
        assert self.zipl.get_template(
            failsafe=True, targettype='CDL'
        ).substitute(
            device='/dev/loop0',
            target_type='CDL',
            blocksize='4096',
            offset=24,
            geometry='2912,15,12',
            default_boot='1',
            bootpath='boot/zipl',
            boot_timeout='200',
            title='LimeJeOS-DASD-ECKD-SLE12_(_VMX_)',
            kernel_file='linux.vmx',
            initrd_file='initrd.vmx',
            boot_options='cio_ignore=all,!ipldev,!condev',
            failsafe_boot_options='x11failsafe cio_ignore=all,!ipldev,!condev'
        )



from mock import patch

from .test_helper import raises

from kiwi.exceptions import KiwiPrivilegesError
from kiwi.privileges import Privileges


class TestPrivileges(object):
    @raises(KiwiPrivilegesError)
    @patch('os.geteuid')
    def test_check_for_root_permiossion_false(self, mock_euid):
        mock_euid.return_value = 1
        Privileges.check_for_root_permissions()

    @patch('os.geteuid')
    def test_check_for_root_permiossion_true(self, mock_euid):
        mock_euid.return_value = 0
        assert Privileges.check_for_root_permissions() is True



# Copyright (c) 2019 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#

from string import Template


class VirtualboxOvfTemplate(object):
    """
    **Generate a OVF file template for a vagrant virtualbox box**

    This class provides a template for virtualbox' ovf configuration file that
    is embedded inside the vagrant box. The template itself was extracted from
    a vagrant box that was build via packer and from a script provided by Neal
    Gompa.
    """

    def __init__(self):
        # note: the OS ID is set to Other Linux 64
        self.OVF_TEMPLATE = """<?xml version="1.0"?>
<Envelope ovf:version="1.0" xml:lang="en-US" xmlns="http://schemas.dmtf.org/ovf/envelope/1" xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/1" xmlns:rasd="http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/CIM_ResourceAllocationSettingData" xmlns:vssd="http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/CIM_VirtualSystemSettingData" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:vbox="http://www.virtualbox.org/ovf/machine">
  <References>
    <File ovf:id="file1" ovf:href="box.vmdk"/>
  </References>
  <DiskSection>
    <Info>List of the virtual disks used in the package</Info>
    <Disk ovf:capacity="${disk_image_capacity}" ovf:diskId="vmdisk1" ovf:fileRef="file1" ovf:format="http://www.vmware.com/interfaces/specifications/vmdk.html#streamOptimized"/>
  </DiskSection>
  <NetworkSection>
    <Info>Logical networks used in the package</Info>
    <Network ovf:name="NAT">
      <Description>Logical network used by this appliance.</Description>
    </Network>
  </NetworkSection>
  <VirtualSystem ovf:id="${vm_name}">
    <Info>A virtual machine</Info>
    <OperatingSystemSection ovf:id="101">
      <Info>The kind of installed guest operating system</Info>
      <Description>${vm_description}</Description>
      <vbox:OSType ovf:required="false">Linux_64</vbox:OSType>
    </OperatingSystemSection>
    <VirtualHardwareSection>
      <Info>Virtual hardware requirements for a virtual machine</Info>
      <System>
        <vssd:ElementName>Virtual Hardware Family</vssd:ElementName>
        <vssd:InstanceID>0</vssd:InstanceID>
        <vssd:VirtualSystemIdentifier>${vm_name}</vssd:VirtualSystemIdentifier>
        <vssd:VirtualSystemType>virtualbox-2.2</vssd:VirtualSystemType>
      </System>
      <Item>
        <rasd:Caption>1 virtual CPU</rasd:Caption>
        <rasd:Description>Number of virtual CPUs</rasd:Description>
        <rasd:ElementName>1 virtual CPU(s)</rasd:ElementName>
        <rasd:InstanceID>1</rasd:InstanceID>
        <rasd:ResourceType>3</rasd:ResourceType>
        <rasd:VirtualQuantity>1</rasd:VirtualQuantity>
      </Item>
      <Item>
        <rasd:AllocationUnits>MegaBytes</rasd:AllocationUnits>
        <rasd:Caption>1024 MB of memory</rasd:Caption>
        <rasd:Description>Memory Size</rasd:Description>
        <rasd:ElementName>1024 MB of memory</rasd:ElementName>
        <rasd:InstanceID>2</rasd:InstanceID>
        <rasd:ResourceType>4</rasd:ResourceType>
        <rasd:VirtualQuantity>1024</rasd:VirtualQuantity>
      </Item>
      <Item>
        <rasd:Address>0</rasd:Address>
        <rasd:Caption>ideController0</rasd:Caption>
        <rasd:Description>IDE Controller</rasd:Description>
        <rasd:ElementName>ideController0</rasd:ElementName>
        <rasd:InstanceID>3</rasd:InstanceID>
        <rasd:ResourceSubType>PIIX4</rasd:ResourceSubType>
        <rasd:ResourceType>5</rasd:ResourceType>
      </Item>
      <Item>
        <rasd:Address>1</rasd:Address>
        <rasd:Caption>ideController1</rasd:Caption>
        <rasd:Description>IDE Controller</rasd:Description>
        <rasd:ElementName>ideController1</rasd:ElementName>
        <rasd:InstanceID>4</rasd:InstanceID>
        <rasd:ResourceSubType>PIIX4</rasd:ResourceSubType>
        <rasd:ResourceType>5</rasd:ResourceType>
      </Item>
      <Item>
        <rasd:Address>0</rasd:Address>
        <rasd:Caption>sataController0</rasd:Caption>
        <rasd:Description>SATA Controller</rasd:Description>
        <rasd:ElementName>sataController0</rasd:ElementName>
        <rasd:InstanceID>5</rasd:InstanceID>
        <rasd:ResourceSubType>AHCI</rasd:ResourceSubType>
        <rasd:ResourceType>20</rasd:ResourceType>
      </Item>
      <Item>
        <rasd:AddressOnParent>3</rasd:AddressOnParent>
        <rasd:AutomaticAllocation>false</rasd:AutomaticAllocation>
        <rasd:Caption>sound</rasd:Caption>
        <rasd:Description>Sound Card</rasd:Description>
        <rasd:ElementName>sound</rasd:ElementName>
        <rasd:InstanceID>6</rasd:InstanceID>
        <rasd:ResourceSubType>ensoniq1371</rasd:ResourceSubType>
        <rasd:ResourceType>35</rasd:ResourceType>
      </Item>
      <Item>
        <rasd:AddressOnParent>0</rasd:AddressOnParent>
        <rasd:Caption>disk1</rasd:Caption>
        <rasd:Description>Disk Image</rasd:Description>
        <rasd:ElementName>disk1</rasd:ElementName>
        <rasd:HostResource>/disk/vmdisk1</rasd:HostResource>
        <rasd:InstanceID>7</rasd:InstanceID>
        <rasd:Parent>5</rasd:Parent>
        <rasd:ResourceType>17</rasd:ResourceType>
      </Item>
      <Item>
        <rasd:AutomaticAllocation>true</rasd:AutomaticAllocation>
        <rasd:Caption>Ethernet adapter on 'NAT'</rasd:Caption>
        <rasd:Connection>NAT</rasd:Connection>
        <rasd:ElementName>Ethernet adapter on 'NAT'</rasd:ElementName>
        <rasd:InstanceID>8</rasd:InstanceID>
        <rasd:ResourceSubType>E1000</rasd:ResourceSubType>
        <rasd:ResourceType>10</rasd:ResourceType>
      </Item>
    </VirtualHardwareSection>
  </VirtualSystem>
</Envelope>
"""

    def get_template(self):
        """
        Return the actual ovf template. The following values must be
        substituted:
        - ``vm_name``: the name of this VM
        - ``disk_image_capacity``: Size of the virtual disk image in GB
        - ``vm_description``: a description of this VM
        """
        return Template(self.OVF_TEMPLATE)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import platform

# project
from kiwi.defaults import Defaults
from kiwi.archive.tar import ArchiveTar
from kiwi.system.setup import SystemSetup
from kiwi.utils.checksum import Checksum
from kiwi.logger import log
from kiwi.system.result import Result
from kiwi.runtime_config import RuntimeConfig

from kiwi.exceptions import (
    KiwiArchiveSetupError
)


class ArchiveBuilder(object):
    """
    **Root archive image builder**

    :param obsject xml_state: Instance of :class:`XMLState`
    :param str target_dir: target directory path name
    :param str root_dir: root directory path name
    :param dict custom_args: Custom processing arguments defined as hash keys:
        * xz_options: string of XZ compression parameters
    """
    def __init__(self, xml_state, target_dir, root_dir, custom_args=None):
        self.root_dir = root_dir
        self.target_dir = target_dir
        self.xml_state = xml_state
        self.requested_archive_type = xml_state.get_build_type_name()
        self.result = Result(xml_state)
        self.system_setup = SystemSetup(
            xml_state=xml_state, root_dir=self.root_dir
        )
        self.filename = self._target_file_for('tar.xz')
        self.checksum = self._target_file_for('md5')
        self.xz_options = custom_args['xz_options'] if custom_args \
            and 'xz_options' in custom_args else None

        self.runtime_config = RuntimeConfig()

    def create(self):
        """
        Create a root archive tarball

        Build a simple XZ compressed root tarball from the image root tree

        Image types which triggers this builder are:

        * image="tbz"

        :return: result

        :rtype: instance of :class:`Result`
        """
        supported_archives = Defaults.get_archive_image_types()
        if self.requested_archive_type not in supported_archives:
            raise KiwiArchiveSetupError(
                'Unknown archive type: %s' % self.requested_archive_type
            )

        if self.requested_archive_type == 'tbz':
            log.info('Creating XZ compressed tar archive')
            archive = ArchiveTar(
                self._target_file_for('tar')
            )
            archive.create_xz_compressed(
                self.root_dir, xz_options=self.xz_options,
                exclude=Defaults.get_exclude_list_for_root_data_sync()
            )
            checksum = Checksum(self.filename)
            log.info('--> Creating archive checksum')
            checksum.md5(self.checksum)
            self.result.verify_image_size(
                self.runtime_config.get_max_size_constraint(),
                self.filename
            )
            self.result.add(
                key='root_archive',
                filename=self.filename,
                use_for_bundle=True,
                compress=False,
                shasum=True
            )
            self.result.add(
                key='root_archive_md5',
                filename=self.checksum,
                use_for_bundle=False
            )
            self.result.add(
                key='image_packages',
                filename=self.system_setup.export_package_list(
                    self.target_dir
                ),
                use_for_bundle=True,
                compress=False,
                shasum=False
            )
            self.result.add(
                key='image_verified',
                filename=self.system_setup.export_package_verification(
                    self.target_dir
                ),
                use_for_bundle=True,
                compress=False,
                shasum=False
            )
        return self.result

    def _target_file_for(self, suffix):
        return ''.join(
            [
                self.target_dir, '/',
                self.xml_state.xml_data.get_name(),
                '.' + platform.machine(),
                '-' + self.xml_state.get_image_version(),
                '.', suffix
            ]
        )



from mock import patch

from .test_helper import raises

from kiwi.system.root_import import RootImport
from kiwi.exceptions import KiwiRootImportError


class TestRootImport(object):
    @patch('kiwi.system.root_import.RootImportOCI')
    def test_docker_import(self, mock_docker_import):
        RootImport('root_dir', 'file:///image.tar.xz', 'docker')
        mock_docker_import.assert_called_once_with(
            'root_dir', 'file:///image.tar.xz',
            custom_args={'archive_transport': 'docker-archive'}
        )

    @patch('kiwi.system.root_import.RootImportOCI')
    def test_oci_import(self, mock_oci_import):
        RootImport('root_dir', 'file:///image.tar.xz', 'oci')
        mock_oci_import.assert_called_once_with(
            'root_dir', 'file:///image.tar.xz',
            custom_args={'archive_transport': 'oci-archive'}
        )

    @raises(KiwiRootImportError)
    def test_not_implemented_import(self):
        RootImport('root_dir', 'file:///image.tar.xz', 'foo')



#!/usr/bin/python3
"""
usage: update_changelog (--since=<reference_file>|--file=<reference_file>)
            [--utc]

arguments:
    --since=<reference_file>
        changes since the latest entry in the reference file
    --utc
        print date/time in UTC
"""
import docopt
import os
import subprocess
import sys
from dateutil import parser
from dateutil import tz

# Commandline arguments
arguments = docopt.docopt(__doc__)

# Latest date of the given reference file
date_reference = None

# List of skipped commits older than date_reference
skip_list = []

# hash of git history log entries
log_data = {}

# raw list of log lines from git history or reference file
log_lines = []

# Author and Date
log_author = None
log_date = None

# changelog header line
log_start = '-' * 67 + os.linesep

# date format for rpm changelog
date_format = '%a %b %d %T %Z %Y'

# commit message
commit_message = []

# Open reference log file
reference_file = arguments['--since'] or arguments['--file']

if arguments['--since']:
    # Read latest date from reference file
    with open(reference_file, 'r') as gitlog:
        # read commit and author
        gitlog.readline()
        gitlog.readline()
        # read date
        latest_date = gitlog.readline().replace('AuthorDate:', '').strip()
        date_reference = parser.parse(latest_date)

    # Read git history since latest entry from reference file
    process = subprocess.Popen(
        [
            'git', 'log', '--no-merges', '--format=fuller',
            '--since="{0}"'.format(latest_date)
        ], stdout=subprocess.PIPE
    )
    for line in iter(process.stdout.readline, b''):
        log_lines.append(line)
else:
    with open(reference_file, 'rb') as gitlog:
        log_lines = gitlog.readlines()

# Iterate over log data and convert to changelog format
for line_data in log_lines:
    line = line_data.decode(encoding='utf-8')
    if line.startswith('commit'):
        if commit_message:
            commit_message.pop(0)
            message_header = commit_message.pop(0).lstrip()
            message_body = []
            for line in commit_message:
                message_line = line.lstrip()
                if not message_line:
                    message_body.append(os.linesep)
                else:
                    message_body.append(
                        '  {0}{1}'.format(message_line, os.linesep)
                    )
            log_data[log_date] = ''.join(
                [
                    log_start,
                    '{0} - {1}{2}{2}'.format(
                        log_date.astimezone(
                            tz.UTC if arguments['--utc'] else tz.tzlocal()
                        ).strftime(date_format), log_author, os.linesep
                    ),
                    '- {0}{1}'.format(
                        message_header, os.linesep
                    )
                ] + message_body
            )
            commit_message = []
    elif line.startswith('Author:'):
        log_author = line.replace('Author:', '').strip()
    elif line.startswith('AuthorDate:'):
        log_date = parser.parse(line.replace('AuthorDate:', '').strip())
    elif line.startswith('Commit:'):
        pass
    elif line.startswith('CommitDate:'):
        pass
    else:
        commit_message.append(line.strip())

# print in changelog format on stdout
for author_date in reversed(sorted(log_data.keys())):
    if date_reference:
        if date_reference < author_date:
            sys.stdout.write(log_data[author_date])
        else:
            skip_list.append(author_date)
    else:
        sys.stdout.write(log_data[author_date])

# print inconsistencies if any on stderr
if skip_list:
    sys.stderr.write(
        'Reference Date: {0}{1}'.format(date_reference, os.linesep)
    )
    for date in skip_list:
        sys.stderr.write(
            '  + Skipped: {0}: past reference{1}'.format(
                date, os.linesep
            )
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from string import Template
from textwrap import dedent


class BootLoaderTemplateIsoLinux(object):
    """
    **isolinux configuraton file templates**
    """
    def __init__(self):
        self.cr = '\n'

        self.install_message = dedent('''
            Welcome !

            Boot_from_Hard_Disk
            Install_${title}
            Failsafe_--_Install_${title}


            Have a lot of fun...
        ''').strip() + self.cr

        self.message = dedent('''
            Welcome !

            ${title}
            Failsafe_${title}


            Have a lot of fun...
        ''').strip() + self.cr

        self.serial = dedent('''
            serial 0 9600
        ''')

        self.header = dedent('''
            # kiwi generated isolinux config file
            implicit 1
            prompt   1
            display isolinux.msg
            default ${default_boot}
        ''').strip() + self.cr

        self.timeout = dedent('''
            timeout  ${boot_timeout}
        ''').strip() + self.cr

        self.ui_theme = dedent('''
            ui gfxboot bootlogo isolinux.msg
            MENU RESOLUTION ${gfxmode}
        ''').strip() + self.cr

        self.ui_plain = dedent('''
            ui menu.c32
        ''').strip() + self.cr

        self.menu_harddisk_entry = dedent('''
            label Boot_from_Hard_Disk
                localboot 0x80
        ''').strip() + self.cr

        self.menu_install_entry_multiboot = dedent('''
            label Install_${title}
                kernel mboot.c32
                append ${hypervisor} --- ${kernel_file} ${boot_options} cdinst=1 showopts --- ${initrd_file} showopts
        ''').strip() + self.cr

        self.menu_install_entry_failsafe_multiboot = dedent('''
            label Failsafe_--_Install_${title}
                kernel mboot.c32
                append ${hypervisor} --- ${kernel_file} ${failsafe_boot_options} cdinst=1 showopts --- ${initrd_file} showopts
        ''').strip() + self.cr

        self.menu_entry_multiboot = dedent('''
            label ${title}
                kernel mboot.c32
                append ${hypervisor} --- ${kernel_file} ${boot_options} showopts --- ${initrd_file} showopts
        ''').strip() + self.cr

        self.menu_entry_failsafe_multiboot = dedent('''
            label Failsafe_--_${title}
                kernel mboot.c32
                append ${hypervisor} --- ${kernel_file} ${failsafe_boot_options} showopts --- ${initrd_file} showopts
        ''').strip() + self.cr

        self.menu_mediacheck_entry_multiboot = dedent('''
            label Mediacheck
                kernel mboot.c32
                append ${hypervisor} --- ${kernel_file} ${boot_options} mediacheck=1 plymouth.enable=0 showopts --- ${initrd_file} showopts
        ''').strip() + self.cr

        self.menu_install_entry = dedent('''
            label Install_${title}
                kernel ${kernel_file}
                append initrd=${initrd_file} ${boot_options} cdinst=1 showopts
        ''').strip() + self.cr

        self.menu_install_entry_failsafe = dedent('''
            label Failsafe_--_Install_${title}
                kernel ${kernel_file}
                append initrd=${initrd_file} ${failsafe_boot_options} cdinst=1 showopts
        ''').strip() + self.cr

        self.menu_entry = dedent('''
            label ${title}
                kernel ${kernel_file}
                append initrd=${initrd_file} ${boot_options} showopts
        ''').strip() + self.cr

        self.menu_entry_failsafe = dedent('''
            label Failsafe_--_${title}
                kernel ${kernel_file}
                append initrd=${initrd_file} ${failsafe_boot_options} showopts
        ''').strip() + self.cr

        self.menu_mediacheck_entry = dedent('''
            label Mediacheck
                kernel ${kernel_file}
                append initrd=${initrd_file} ${boot_options} mediacheck=1 plymouth.enable=0 showopts
        ''').strip() + self.cr

    def get_install_message_template(self):
        """
        Bootloader template for text message file in install mode.
        isolinux displays this as menu if no graphics mode can be
        initialized

        :return: instance of :class:`Template`

        :rtype: Template
        """
        return Template(self.install_message)

    def get_message_template(self):
        """
        Bootloader template for text message file. isolinux
        displays this as menu if no graphics mode can be initialized

        :return: instance of :class:`Template`

        :rtype: Template
        """
        return Template(self.message)

    def get_template(
        self, failsafe=True, with_theme=True, terminal=None, checkiso=False
    ):
        """
        Bootloader configuration template for live media

        :param bool failsafe: with failsafe true|false
        :param bool with_theme: with graphics theme true|false

        :return: instance of :class:`Template`

        :rtype: Template
        """
        template_data = self.header
        template_data += self.timeout
        if terminal and 'serial' in terminal:
            template_data += self.serial
            with_theme = False
        if with_theme:
            template_data += self.ui_theme
        else:
            template_data += self.ui_plain
        template_data += self.menu_entry
        if failsafe:
            template_data += self.menu_entry_failsafe
        template_data += self.menu_harddisk_entry
        if checkiso:
            template_data += self.menu_mediacheck_entry
        return Template(template_data)

    def get_multiboot_template(
        self, failsafe=True, with_theme=True, terminal=None, checkiso=False
    ):
        """
        Bootloader configuration template for live media with
        hypervisor, e.g Xen dom0

        :param bool failsafe: with failsafe true|false
        :param bool with_theme: with graphics theme true|false

        :return: instance of :class:`Template`

        :rtype: Template
        """
        template_data = self.header
        template_data += self.timeout
        if terminal and 'serial' in terminal:
            template_data += self.serial
            with_theme = False
        if with_theme:
            template_data += self.ui_theme
        else:
            template_data += self.ui_plain
        template_data += self.menu_entry_multiboot
        if failsafe:
            template_data += self.menu_entry_failsafe_multiboot
        template_data += self.menu_harddisk_entry
        if checkiso:
            template_data += self.menu_mediacheck_entry_multiboot
        return Template(template_data)

    def get_install_template(
        self, failsafe=True, with_theme=True, terminal=None, with_timeout=True
    ):
        """
        Bootloader configuration template for install media

        :param bool failsafe: with failsafe true|false
        :param bool with_theme: with graphics theme true|false

        :return: instance of :class:`Template`

        :rtype: Template
        """
        template_data = self.header
        if with_timeout:
            template_data += self.timeout
        if terminal and 'serial' in terminal:
            template_data += self.serial
            with_theme = False
        if with_theme:
            template_data += self.ui_theme
        else:
            template_data += self.ui_plain
        template_data += self.menu_harddisk_entry
        template_data += self.menu_install_entry
        if failsafe:
            template_data += self.menu_install_entry_failsafe
        return Template(template_data)

    def get_multiboot_install_template(
        self, failsafe=True, with_theme=True, terminal=None, with_timeout=True
    ):
        """
        Bootloader configuration template for install media with
        hypervisor, e.g Xen dom0

        :param bool failsafe: with failsafe true|false
        :param bool with_theme: with graphics theme true|false

        :return: instance of :class:`Template`

        :rtype: Template
        """
        template_data = self.header
        if with_timeout:
            template_data += self.timeout
        if terminal and 'serial' in terminal:
            template_data += self.serial
            with_theme = False
        if with_theme:
            template_data += self.ui_theme
        else:
            template_data += self.ui_plain
        template_data += self.menu_harddisk_entry
        template_data += self.menu_install_entry_multiboot
        if failsafe:
            template_data += self.menu_install_entry_failsafe_multiboot
        return Template(template_data)



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import copy

# project
from kiwi.logger import log
from kiwi.utils.sync import DataSync
from kiwi.mount_manager import MountManager

from kiwi.exceptions import (
    KiwiFileSystemSyncError
)


class FileSystemBase(object):
    """
    **Implements base class for filesystem interface**

    :param object device_provider:
        Instance of a class based on DeviceProvider
        required for filesystems which needs a block device for
        creation. In most cases the DeviceProvider is a LoopDevice
    :param string root_dir: root directory path name
    :param dict custom_args: custom filesystem arguments
    """
    def __init__(self, device_provider, root_dir=None, custom_args=None):
        # filesystems created with a block device stores the mountpoint
        # here. The file name of the file containing the filesystem is
        # stored in the device_provider if the filesystem is represented
        # as a file there
        self.filesystem_mount = None

        # bind the block device providing class instance to this object.
        # This is done to guarantee the correct destructor order when
        # the device should be released. This is only required if the
        # filesystem required a block device to become created
        self.device_provider = device_provider

        self.root_dir = root_dir

        # filesystems created without a block device stores the result
        # filesystem file name here
        self.filename = None

        self.custom_args = {}
        self.post_init(custom_args)

    def post_init(self, custom_args):
        """
        Post initialization method

        Store dictionary of custom arguments if not empty. This
        overrides the default custom argument hash

        :param dict custom_args: custom arguments

            .. code:: python

                {
                    'create_options': ['option'],
                    'mount_options': ['option'],
                    'meta_data': {
                        'key': 'value'
                    }
                }
        """
        if custom_args:
            self.custom_args = copy.deepcopy(custom_args)

        if 'create_options' not in self.custom_args:
            self.custom_args['create_options'] = []

        if 'meta_data' not in self.custom_args:
            self.custom_args['meta_data'] = {}

        if 'mount_options' not in self.custom_args:
            self.custom_args['mount_options'] = []

    def create_on_device(self, label=None):
        """
        Create filesystem on block device

        Implement in specialized filesystem class for filesystems which
        requires a block device for creation, e.g ext4.

        :param string label: label name
        """
        raise NotImplementedError

    def create_on_file(self, filename, label=None, exclude=None):
        """
        Create filesystem from root data tree

        Implement in specialized filesystem class for filesystems which
        requires a data tree for creation, e.g squashfs.

        :param string filename: result file path name
        :param string label: label name
        :param list exclude: list of exclude dirs/files
        """
        raise NotImplementedError

    def sync_data(self, exclude=None):
        """
        Copy root data tree into filesystem

        :param list exclude: list of exclude dirs/files
        """
        if not self.root_dir:
            raise KiwiFileSystemSyncError(
                'no root directory specified'
            )
        if not os.path.exists(self.root_dir):
            raise KiwiFileSystemSyncError(
                'given root directory %s does not exist' % self.root_dir
            )
        self.filesystem_mount = MountManager(
            device=self.device_provider.get_device()
        )
        self.filesystem_mount.mount(
            self.custom_args['mount_options']
        )
        data = DataSync(
            self.root_dir, self.filesystem_mount.mountpoint
        )
        data.sync_data(
            options=['-a', '-H', '-X', '-A', '--one-file-system'],
            exclude=exclude
        )
        self.filesystem_mount.umount()

    def __del__(self):
        if self.filesystem_mount:
            log.info('Cleaning up %s instance', type(self).__name__)
            self.filesystem_mount.umount()



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from .grub2 import BootLoaderInstallGrub2
from .zipl import BootLoaderInstallZipl

from ...exceptions import (
    KiwiBootLoaderInstallSetupError
)


class BootLoaderInstall(object):
    """
    **BootLoaderInstall Factory**

    :param string name: bootloader name
    :param string root_dir: root directory path name
    :param object device_provider: instance of :class:`DeviceProvider`
    :param dict custom_args: custom arguments dictionary
    """
    def __new__(self, name, root_dir, device_provider, custom_args=None):
        if name == 'grub2':
            return BootLoaderInstallGrub2(
                root_dir, device_provider, custom_args
            )
        elif name == 'grub2_s390x_emu':
            return BootLoaderInstallZipl(
                root_dir, device_provider, custom_args
            )
        else:
            raise KiwiBootLoaderInstallSetupError(
                'Support for %s bootloader installation not implemented' % name
            )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import re
from tempfile import NamedTemporaryFile
from collections import (
    namedtuple,
    OrderedDict
)

# project
from kiwi.defaults import Defaults
from kiwi.iso_tools.base import IsoToolsBase
from kiwi.utils.command_capabilities import CommandCapabilities
from kiwi.path import Path
from kiwi.command import Command
from kiwi.exceptions import KiwiIsoToolError


class IsoToolsCdrTools(IsoToolsBase):
    """
    **cdrkit/cdrtools wrapper class**

    Implementation of Parameter API for iso creation tools using
    the cdrkit/cdrtools projects. Addressed here are the option
    compatible tools mkisofs and genisoimage
    """
    def has_iso_hybrid_capability(self):
        """
        Indicate if the iso tool has the capability to embed a
        partition table into the iso such that it can be
        used as both; an iso and a disk

        :return: True or False

        :rtype: bool
        """
        return False

    def get_tool_name(self):
        """
        There are tools by J.Schilling and tools from the community
        Depending on what is installed a decision needs to be made.
        mkisofs is preferred over genisoimage

        :raises KiwiIsoToolError: if no iso creation tool is found
        :return: tool name

        :rtype: str
        """
        iso_creation_tools = ['mkisofs', 'genisoimage']
        for tool in iso_creation_tools:
            tool_found = Path.which(tool)
            if tool_found:
                return tool_found

        raise KiwiIsoToolError(
            'No iso creation tool found, searched for: %s'.format(
                iso_creation_tools
            )
        )

    def init_iso_creation_parameters(self, custom_args=None):
        """
        Create a set of standard parameters

        :param list custom_args: custom ISO creation args
        """
        efi_mode = False
        if custom_args:
            if custom_args.get('efi_mode'):
                efi_mode = True
            if 'mbr_id' in custom_args:
                self.iso_parameters += [
                    '-A', custom_args['mbr_id']
                ]
            if 'publisher' in custom_args:
                self.iso_parameters += [
                    '-publisher', custom_args['publisher']
                ]
            if 'preparer' in custom_args:
                self.iso_parameters += [
                    '-p', custom_args['preparer']
                ]
            if 'volume_id' in custom_args:
                self.iso_parameters += [
                    '-V', custom_args['volume_id']
                ]
            if 'udf' in custom_args:
                self.iso_parameters += [
                    '-iso-level', '3', '-udf'
                ]
        catalog_file = self.boot_path + '/boot.catalog'
        self.iso_parameters += [
            '-R', '-J', '-f', '-pad', '-joliet-long',
            '-sort', self._create_sortfile(),
            '-no-emul-boot', '-boot-load-size', '4', '-boot-info-table',
            '-hide', catalog_file,
            '-hide-joliet', catalog_file,
        ]
        if efi_mode:
            loader_file = os.sep.join(
                [
                    self.boot_path, 'loader',
                    Defaults.get_isolinux_bios_grub_loader()
                ]
            )
        else:
            loader_file = self.boot_path + '/loader/isolinux.bin'
        self.iso_loaders += [
            '-b', loader_file, '-c', catalog_file
        ]

    def add_efi_loader_parameters(self):
        """
        Add ISO creation parameters to embed the EFI loader

        In order to boot the ISO from EFI, the EFI binary is added as
        alternative loader to the ISO creation parameter list. The
        EFI binary must be included into a fat filesystem in order
        to become recognized by the firmware. For details about this
        file refer to _create_embedded_fat_efi_image() from
        bootloader/config/grub2.py
        """
        loader_file = self.boot_path + '/efi'
        if os.path.exists(os.sep.join([self.source_dir, loader_file])):
            self.iso_loaders.append('-eltorito-alt-boot')
            iso_tool = self.get_tool_name()
            if iso_tool and CommandCapabilities.has_option_in_help(
                iso_tool, '-eltorito-platform', raise_on_error=False
            ):
                self.iso_loaders += ['-eltorito-platform', 'efi']
            self.iso_loaders += [
                '-b', loader_file, '-no-emul-boot', '-joliet-long'
            ]
            loader_file_512_byte_blocks = os.path.getsize(
                os.sep.join([self.source_dir, loader_file])
            ) / 512
            # boot-load-size is stored in a 16bit range, thus we only
            # set the value if it fits into that range
            if loader_file_512_byte_blocks <= 0xffff:
                self.iso_loaders.append(
                    '-boot-load-size'
                )
                self.iso_loaders.append(
                    format(int(loader_file_512_byte_blocks))
                )

    def create_iso(self, filename, hidden_files=None):
        """
        Creates the iso file with the given filename using cdrtools

        :param str filename: output filename
        :param list hidden_files: list of hidden files
        """
        hidden_files_parameters = []
        if hidden_files:
            for hidden_file in hidden_files:
                hidden_files_parameters.append('-hide')
                hidden_files_parameters.append(hidden_file)
                hidden_files_parameters.append('-hide-joliet')
                hidden_files_parameters.append(hidden_file)
        iso_parameters = \
            hidden_files_parameters + \
            self.iso_parameters + \
            self.iso_loaders
        Command.run(
            [
                self.get_tool_name()
            ] + iso_parameters + [
                '-o', filename, self.source_dir
            ]
        )

    def list_iso(self, isofile):
        """
        List contents of an ISO image

        :param str isofile: path to the ISO file

        :return: formatted isoinfo result

        :rtype: dict
        """
        listing_type = namedtuple(
            'listing_type', ['name', 'filetype', 'start']
        )
        listing = Command.run(
            [self._get_isoinfo_tool(), '-R', '-l', '-i', isofile]
        )
        listing_result = {}
        for line in listing.output.split(os.linesep):
            iso_entry = re.search(
                '.*(-[-rwx]{9}).*\s\[\s*(\d+)(\s+\d+)?\]\s+(.*?)\s*$', line
            )
            if iso_entry:
                entry_type = iso_entry.group(1)
                entry_name = iso_entry.group(4)
                entry_addr = int(iso_entry.group(2))
                listing_result[entry_addr] = listing_type(
                    name=entry_name,
                    filetype=entry_type,
                    start=entry_addr
                )
        return OrderedDict(
            sorted(listing_result.items())
        )

    def _get_isoinfo_tool(self):
        """
        There are tools by J.Schilling and tools from the community
        This method searches in all paths which could provide an
        isoinfo tool. The first match makes the decision

        :raises KiwiIsoToolError: if no isoinfo tool found
        :return: the isoinfo tool to use

        :rtype: str
        """
        alternative_lookup_paths = ['/usr/lib/genisoimage']
        isoinfo = Path.which('isoinfo', alternative_lookup_paths)
        if isoinfo:
            return isoinfo

        raise KiwiIsoToolError(
            'No isoinfo tool found, searched in PATH: %s and %s' %
            (os.environ.get('PATH'), alternative_lookup_paths)
        )

    def _create_sortfile(self):
        """
        Create isolinux sort file

        :return: iso sort file name

        :rtype: str
        """
        self.iso_sortfile = NamedTemporaryFile()
        catalog_file = \
            self.source_dir + '/' + self.boot_path + '/boot.catalog'
        loader_file = \
            self.source_dir + '/' + self.boot_path + '/loader/isolinux.bin'
        with open(self.iso_sortfile.name, 'w') as sortfile:
            sortfile.write('%s 3\n' % catalog_file)
            sortfile.write('%s 2\n' % loader_file)

            boot_files = list(os.walk(self.source_dir + '/' + self.boot_path))
            boot_files += list(os.walk(self.source_dir + '/EFI'))
            for basedir, dirnames, filenames in sorted(boot_files):
                for entry in sorted(dirnames + filenames):
                    if entry in filenames and entry == 'efi':
                        sortfile.write('%s/%s 1000001\n' % (basedir, entry))
                    else:
                        sortfile.write('%s/%s 1\n' % (basedir, entry))

            sortfile.write(
                '%s/%s 1000000\n' % (self.source_dir, 'header_end')
            )
        return self.iso_sortfile.name



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
from tempfile import mkdtemp
from tempfile import NamedTemporaryFile
import platform
import shutil

# project
from kiwi.bootloader.config import BootLoaderConfig
from kiwi.filesystem import FileSystem
from kiwi.filesystem.isofs import FileSystemIsoFs
from kiwi.filesystem.setup import FileSystemSetup
from kiwi.storage.loop_device import LoopDevice
from kiwi.boot.image import BootImageDracut
from kiwi.system.size import SystemSize
from kiwi.system.setup import SystemSetup
from kiwi.firmware import FirmWare
from kiwi.defaults import Defaults
from kiwi.path import Path
from kiwi.system.result import Result
from kiwi.iso_tools.iso import Iso
from kiwi.system.identifier import SystemIdentifier
from kiwi.system.kernel import Kernel
from kiwi.logger import log
from kiwi.runtime_config import RuntimeConfig
from kiwi.iso_tools.base import IsoToolsBase

from kiwi.exceptions import (
    KiwiLiveBootImageError
)


class LiveImageBuilder(object):
    """
    **Live image builder**

    :param object xml_state: instance of :class:`XMLState`
    :param str target_dir: target directory path name
    :param str root_dir: root directory path name
    :param dict custom_args: Custom processing arguments
    """
    def __init__(self, xml_state, target_dir, root_dir, custom_args=None):
        self.media_dir = None
        self.live_container_dir = None
        self.arch = platform.machine()
        if self.arch == 'i686' or self.arch == 'i586':
            self.arch = 'ix86'
        self.root_dir = root_dir
        self.target_dir = target_dir
        self.xml_state = xml_state
        self.live_type = xml_state.build_type.get_flags()
        self.volume_id = xml_state.build_type.get_volid() or \
            Defaults.get_volume_id()
        self.mbrid = SystemIdentifier()
        self.mbrid.calculate_id()
        self.publisher = xml_state.build_type.get_publisher() or \
            Defaults.get_publisher()
        self.custom_args = custom_args

        if not self.live_type:
            self.live_type = Defaults.get_default_live_iso_type()

        self.boot_image = BootImageDracut(
            xml_state, target_dir, self.root_dir
        )
        self.firmware = FirmWare(
            xml_state
        )
        self.system_setup = SystemSetup(
            xml_state=xml_state, root_dir=self.root_dir
        )
        self.isoname = ''.join(
            [
                target_dir, '/',
                xml_state.xml_data.get_name(),
                '.' + platform.machine(),
                '-' + xml_state.get_image_version(),
                '.iso'
            ]
        )
        self.result = Result(xml_state)
        self.runtime_config = RuntimeConfig()

    def create(self):
        """
        Build a bootable hybrid live ISO image

        Image types which triggers this builder are:

        * image="iso"

        :raises KiwiLiveBootImageError: if no kernel or hipervisor is found
            in boot image tree
        :return: result

        :rtype: instance of :class:`Result`
        """
        # media dir to store CD contents
        self.media_dir = mkdtemp(
            prefix='live-media.', dir=self.target_dir
        )

        # unpack cdroot user files to media dir
        self.system_setup.import_cdroot_files(self.media_dir)

        rootsize = SystemSize(self.media_dir)

        # custom iso metadata
        log.info('Using following live ISO metadata:')
        log.info('--> Application id: {0}'.format(self.mbrid.get_id()))
        log.info('--> Publisher: {0}'.format(Defaults.get_publisher()))
        log.info('--> Volume id: {0}'.format(self.volume_id))
        custom_iso_args = {
            'meta_data': {
                'publisher': self.publisher,
                'preparer': Defaults.get_preparer(),
                'volume_id': self.volume_id,
                'mbr_id': self.mbrid.get_id(),
                'efi_mode': self.firmware.efi_mode()
            }
        }

        log.info(
            'Setting up live image bootloader configuration'
        )
        if self.firmware.efi_mode():
            # setup bootloader config to boot the ISO via EFI
            # This also embedds an MBR and the respective BIOS modules
            # for compat boot. The complete bootloader setup will be
            # based on grub
            bootloader_config = BootLoaderConfig(
                'grub2', self.xml_state, root_dir=self.root_dir,
                boot_dir=self.media_dir, custom_args={
                    'grub_directory_name':
                        Defaults.get_grub_boot_directory_name(self.root_dir)
                }
            )
            bootloader_config.setup_live_boot_images(
                mbrid=self.mbrid, lookup_path=self.root_dir
            )
        else:
            # setup bootloader config to boot the ISO via isolinux.
            # This allows for booting on x86 platforms in BIOS mode
            # only.
            bootloader_config = BootLoaderConfig(
                'isolinux', self.xml_state, root_dir=self.root_dir,
                boot_dir=self.media_dir
            )
        IsoToolsBase.setup_media_loader_directory(
            self.boot_image.boot_root_directory, self.media_dir,
            bootloader_config.get_boot_theme()
        )
        bootloader_config.setup_live_image_config(
            mbrid=self.mbrid
        )
        bootloader_config.write()

        # call custom editbootconfig script if present
        self.system_setup.call_edit_boot_config_script(
            filesystem='iso:{0}'.format(self.media_dir), boot_part_id=1,
            working_directory=self.root_dir
        )

        # prepare dracut initrd call
        self.boot_image.prepare()

        # create dracut initrd for live image
        log.info('Creating live ISO boot image')
        self._create_dracut_live_iso_config()
        self.boot_image.create_initrd(self.mbrid)

        # setup kernel file(s) and initrd in ISO boot layout
        log.info('Setting up kernel file(s) and boot image in ISO boot layout')
        self._setup_live_iso_kernel_and_initrd()

        # calculate size and decide if we need UDF
        if rootsize.accumulate_mbyte_file_sizes() > 4096:
            log.info('ISO exceeds 4G size, using UDF filesystem')
            custom_iso_args['meta_data']['udf'] = True

        # pack system into live boot structure as expected by dracut
        log.info(
            'Packing system into dracut live ISO type: {0}'.format(
                self.live_type
            )
        )
        root_filesystem = Defaults.get_default_live_iso_root_filesystem()
        filesystem_custom_parameters = {
            'mount_options': self.xml_state.get_fs_mount_option_list(),
            'create_options': self.xml_state.get_fs_create_option_list()
        }
        filesystem_setup = FileSystemSetup(
            self.xml_state, self.root_dir
        )
        root_image = NamedTemporaryFile()
        loop_provider = LoopDevice(
            root_image.name,
            filesystem_setup.get_size_mbytes(root_filesystem),
            self.xml_state.build_type.get_target_blocksize()
        )
        loop_provider.create()
        live_filesystem = FileSystem(
            name=root_filesystem,
            device_provider=loop_provider,
            root_dir=self.root_dir + os.sep,
            custom_args=filesystem_custom_parameters
        )
        live_filesystem.create_on_device()
        log.info(
            '--> Syncing data to {0} root image'.format(root_filesystem)
        )
        live_filesystem.sync_data(
            Defaults.get_exclude_list_for_root_data_sync()
        )
        log.info('--> Creating squashfs container for root image')
        self.live_container_dir = mkdtemp(
            prefix='live-container.', dir=self.target_dir
        )
        Path.create(self.live_container_dir + '/LiveOS')
        shutil.copy(
            root_image.name, self.live_container_dir + '/LiveOS/rootfs.img'
        )
        live_container_image = FileSystem(
            name='squashfs',
            device_provider=None,
            root_dir=self.live_container_dir
        )
        container_image = NamedTemporaryFile()
        live_container_image.create_on_file(
            container_image.name
        )
        Path.create(self.media_dir + '/LiveOS')
        shutil.copy(
            container_image.name, self.media_dir + '/LiveOS/squashfs.img'
        )

        # create iso filesystem from media_dir
        log.info('Creating live ISO image')
        iso_image = FileSystemIsoFs(
            device_provider=None, root_dir=self.media_dir,
            custom_args=custom_iso_args
        )
        iso_image.create_on_file(self.isoname)

        # include metadata for checkmedia tool
        if self.xml_state.build_type.get_mediacheck() is True:
            Iso.set_media_tag(self.isoname)

        self.result.verify_image_size(
            self.runtime_config.get_max_size_constraint(),
            self.isoname
        )
        self.result.add(
            key='live_image',
            filename=self.isoname,
            use_for_bundle=True,
            compress=False,
            shasum=True
        )
        self.result.add(
            key='image_packages',
            filename=self.system_setup.export_package_list(
                self.target_dir
            ),
            use_for_bundle=True,
            compress=False,
            shasum=False
        )
        self.result.add(
            key='image_verified',
            filename=self.system_setup.export_package_verification(
                self.target_dir
            ),
            use_for_bundle=True,
            compress=False,
            shasum=False
        )
        return self.result

    def _create_dracut_live_iso_config(self):
        live_config_file = self.root_dir + '/etc/dracut.conf.d/02-livecd.conf'
        omit_modules = [
            'kiwi-dump', 'kiwi-overlay', 'kiwi-repart', 'kiwi-lib', 'multipath'
        ]
        live_config = [
            'add_dracutmodules+=" {0} pollcdrom "'.format(
                Defaults.get_live_dracut_module_from_flag(self.live_type)
            ),
            'omit_dracutmodules+=" {0} "'.format(' '.join(omit_modules)),
            'hostonly="no"',
            'dracut_rescue_image="no"'
        ]
        with open(live_config_file, 'w') as config:
            for entry in live_config:
                config.write(entry + os.linesep)

    def _setup_live_iso_kernel_and_initrd(self):
        """
        Copy kernel and initrd from the root tree into the iso boot structure
        """
        boot_path = ''.join(
            [self.media_dir, '/boot/', self.arch, '/loader']
        )
        Path.create(boot_path)

        # Move kernel files to iso filesystem structure
        kernel = Kernel(self.boot_image.boot_root_directory)
        if kernel.get_kernel():
            kernel.copy_kernel(boot_path, '/linux')
        else:
            raise KiwiLiveBootImageError(
                'No kernel in boot image tree {0} found'.format(
                    self.boot_image.boot_root_directory
                )
            )
        if self.xml_state.is_xen_server():
            if kernel.get_xen_hypervisor():
                kernel.copy_xen_hypervisor(boot_path, '/xen.gz')
            else:
                raise KiwiLiveBootImageError(
                    'No hypervisor in boot image tree {0} found'.format(
                        self.boot_image.boot_root_directory
                    )
                )

        # Move initrd to iso filesystem structure
        if os.path.exists(self.boot_image.initrd_filename):
            shutil.move(
                self.boot_image.initrd_filename, boot_path + '/initrd'
            )
        else:
            raise KiwiLiveBootImageError(
                'No boot image {0} in boot image tree {1} found'.format(
                    self.boot_image.initrd_filename,
                    self.boot_image.boot_root_directory
                )
            )

    def __del__(self):
        if self.media_dir or self.live_container_dir:
            log.info(
                'Cleaning up {0} instance'.format(type(self).__name__)
            )
            if self.media_dir:
                Path.wipe(self.media_dir)
            if self.live_container_dir:
                Path.wipe(self.live_container_dir)
